# -*- coding: utf-8 -*-
'''
Created on 2020-11-16

@author: zhys513(254851907@qq.com)
''' 
from opspipe.app.core.logging.logging import LoggerFactory
logger = LoggerFactory('test').logger
  
logger.info(1111)