#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .utils import read_txts,read_txt,read_json
from .utils import load_jsons
from .utils import make_zip,un_zip
from .utils import copy_dir,copy_dirs
from .utils import clean_text
from .base64utils import base64_to_file,base64_to_str